/*****************************************************************************
 * Copyright (c) 2015, 2016 CEA LIST, Christian W. Damus, and others.
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *   CEA LIST - Initial API and implementation
 *   Christian W. Damus - bugs 490859, 492368
 *   
 *****************************************************************************/
package org.eclipse.papyrusrt.umlrt.core.utils;

import static org.eclipse.papyrusrt.umlrt.core.types.IUMLRTElementTypes.EXTERNAL_BEHAVIOR_PORT_ID;
import static org.eclipse.papyrusrt.umlrt.core.types.IUMLRTElementTypes.INTERNAL_BEHAVIOR_PORT_ID;
import static org.eclipse.papyrusrt.umlrt.core.types.IUMLRTElementTypes.RELAY_PORT_ID;
import static org.eclipse.papyrusrt.umlrt.core.types.IUMLRTElementTypes.SERVICE_ACCESS_POINT_ID;
import static org.eclipse.papyrusrt.umlrt.core.types.IUMLRTElementTypes.SERVICE_PROVISION_POINT_ID;

import org.eclipse.gmf.runtime.emf.type.core.ElementTypeRegistry;
import org.eclipse.gmf.runtime.emf.type.core.IElementType;
import org.eclipse.gmf.runtime.emf.type.core.IHintedType;
import org.eclipse.papyrusrt.umlrt.core.types.ElementTypeUtils;
import org.eclipse.papyrusrt.umlrt.core.types.IUMLRTElementTypes;
import org.eclipse.papyrusrt.umlrt.core.types.UMLRTElementTypesEnumerator;

public enum RTPortKindEnum implements Comparable<RTPortKindEnum> {

	/**
	 * 
	 */
	EXTERNAL(0, "External Behavior"),

	/**
	 * 
	 */
	INTERNAL(1, "Internal Behavior"),

	/**
	 *
	 */
	RELAY(2, "Relay"),

	/**
	 * 
	 */
	SAP(3, "SAP"),

	/**
	 * 
	 */
	SPP(4, "SPP");


	private final String label;
	private final int index;

	public String getLabel() {

		return label;
	}

	public int getIndex() {

		return index;
	}



	private RTPortKindEnum(int index, String label) {
		this.label = label;
		this.index = index;

	}

	@Override
	public String toString() {
		return label;
	}

	/**
	 * Queries the kind of <tt>RTPort</tt> indicated by an element-type, if it is some kind
	 * of <tt>RTPort</tt> type.
	 * 
	 * @param elementType
	 *            an element-type
	 * @return the corresponding port kind, or {@code null} if the element-type is not a
	 *         kind of <tt>RTPort</tt>
	 */
	public static RTPortKindEnum of(IElementType elementType) {
		RTPortKindEnum result = null;

		if ((elementType != null) && ElementTypeUtils.isTypeCompatible(elementType, UMLRTElementTypesEnumerator.RT_PORT)) {
			// Which specific kind is it?
			if (ElementTypeUtils.isTypeCompatible(elementType, ElementTypeRegistry.getInstance().getType(INTERNAL_BEHAVIOR_PORT_ID))) {
				result = RTPortKindEnum.INTERNAL;
			} else if (ElementTypeUtils.isTypeCompatible(elementType, ElementTypeRegistry.getInstance().getType(EXTERNAL_BEHAVIOR_PORT_ID))) {
				result = RTPortKindEnum.EXTERNAL;
			} else if (ElementTypeUtils.isTypeCompatible(elementType, ElementTypeRegistry.getInstance().getType(SERVICE_PROVISION_POINT_ID))) {
				result = RTPortKindEnum.SPP;
			} else if (ElementTypeUtils.isTypeCompatible(elementType, ElementTypeRegistry.getInstance().getType(SERVICE_ACCESS_POINT_ID))) {
				result = RTPortKindEnum.SAP;
			} else if (ElementTypeUtils.isTypeCompatible(elementType, ElementTypeRegistry.getInstance().getType(RELAY_PORT_ID))) {
				result = RTPortKindEnum.RELAY;
			}
		}

		return result;
	}

	/**
	 * Converts me to my corresponding {@linkplain IElementType element-type}.
	 * 
	 * @return my element-type
	 */
	public IHintedType toElementType() {
		IElementType result;

		switch (this) {
		case EXTERNAL:
			result = ElementTypeRegistry.getInstance().getType(IUMLRTElementTypes.EXTERNAL_BEHAVIOR_PORT_ID);
			break;
		case INTERNAL:
			result = ElementTypeRegistry.getInstance().getType(IUMLRTElementTypes.INTERNAL_BEHAVIOR_PORT_ID);
			break;
		case RELAY:
			result = ElementTypeRegistry.getInstance().getType(IUMLRTElementTypes.RELAY_PORT_ID);
			break;
		case SAP:
			result = ElementTypeRegistry.getInstance().getType(IUMLRTElementTypes.SERVICE_ACCESS_POINT_ID);
			break;
		case SPP:
			result = ElementTypeRegistry.getInstance().getType(IUMLRTElementTypes.SERVICE_PROVISION_POINT_ID);
			break;
		default:
			result = UMLRTElementTypesEnumerator.RT_PORT;
			break;
		}

		// The registered port types are all hinted
		return (IHintedType) result;
	}
}
